;;; Javier Oviedo's Emacs init file.
;;; joviedo@glue.umd.edu
;;; 
;;; This file contains my emacs customizations. Most of this file I have 
;;; borrowed from other files, but some(very little) I wrote myself.
;;; gnu.help.emacs has been an invaluable resource.
;;;
;;; I have "tried" to organize this file into managable and logical sections,
;;; but there still seems to be a little over lap. I am now considering
;;; breaking this file up into seperate files according to the sections that
;;; I have created.
;;;
;;; Anyway, I am always trying to improve this file so please feel free to
;;; e-mail any suggestions. Again, I can only take true credit for about
;;; 5-10% of this file. The other 90-95% has been obtained through other
;;; peoples .emacs files and newsgroups.

;;; LAST MODIFIED ON:  Wed 19-Jul-2000 11:08:36
;;; LAST MODIFIED BY:  Javier Oviedo




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; various setups

;;; Set the load path
(setq load-path (cons "~/site-lisp" load-path))
(setq load-path (cons "~/" load-path))
(setq enable-local-eval t)
;;;

;;; Control window size, position, and font.
;; (set-frame-height (selected-frame) 40)
;; (set-frame-width (selected-frame) 80)
;; (set-frame-position (selected-frame) 10 30)
;; (set-default-font "-*-courier-normal-r-*-*-13-*-*-*-c-*-*-iso8859-1")
;; (set-face-font 'italic "7X14")
;;;

;;; Use spaces instead of tabs to indent
;; (setq indent-tabs-mode nil)
;;;

;;; return to same line on a scroll back
(setq scroll-preserve-screen-position t)
;;;

;;; Have the titlebar contain name of current buffer
;(setq frame-title-format "%b")
(setq frame-title-format '("%b" (buffer-file-name ": %f")))
;;;

;;; Paste at cursor NOT mouse pointer position
(setq mouse-yank-at-point t)
;;;

;;; Extended video modes; these are for ATi Mach32 SVGA
(setq screen-dimensions-25x132 35)
(setq screen-dimensions-44x132 51)
;;;

;;; Type 'y' instead of 'yes'
(fset 'yes-or-no-p 'y-or-n-p)
;;;

;;; Turn off backup files.
(setq make-backup-files nil)
;;;

;;; Display the time on modeline
(condition-case err
    (display-time)
  (error
   (message "Cannot display time %s" (cdr err))))
;; ;;;

;;; Display the column number on modeline
(condition-case err
    (column-number-mode t)
   (error
        (message "Cannot display Column Number %s" (cdr err))))
;;;

;;; This is to not display the initial message (which says to a novice
;;; user what to do first if he/she is confused).
(setq inhibit-startup-message t)
;;;

;;; ...and this inhibits the startup blurb in the echo area.
(setq inhibit-startup-echo-area-message "john_doe")
;;;

;;; This disables down-arrow and C-n at the end of a buffer from adding
;;; a new line to that buffer.
(setq next-line-add-newlines nil)
;;;

;;; Overlap between window-fulls when scrolling by pages
(setq next-screen-context-lines 2)
;;;

;;; Turn on auto-fill. 
(setq auto-fill-mode t)
;;;

;;; Get rid of old versions of files
(setq delete-old-versions t)
;;;

;;; Highlight during query
(setq query-replace-highlight t)
;;;

;;; Highlight incremental search
(setq search-highlight t)
;;;

;;; SCroll stuff
(setq scroll-conservatively 5)
(setq scroll-step 1)
;;;

;;;
(setq-default transient-mark-mode nil)
;;;

;;; Set for visual bell
(setq visible-bell t)
;;;

;;; Line number mode true
(setq line-number-mode t)
;;;

;;; Start off in "C:/" dir... for use with pc-emacs
;; (cd "C:/")
;;;

;;; Show current function in modeline
;; (which-func-mode t)
;;;

;;; Make searches case insensitive
(setq case-fold-search t)
;;;

;;; Display time in the mode line
;; (setq display-time-day-and-date t)
;;;

;;; Modeline stuff
(setq default-mode-line-format
      '("-"
       mode-line-mule-info
       mode-line-modified
       mode-line-frame-identification
       mode-line-buffer-identification
       " "
       global-mode-string
       "   %[(" mode-name mode-line-process minor-mode-alist "%n" ")%]--"
       (which-func-mode
         ("" which-func-format "--"))
       (line-number-mode "L%l--")
       (column-number-mode "C%c--")
       (-3 . "%p")
       "-%-"))
;;;

;;; Automatically makes the matching paren stand out in color.
(condition-case err
    (show-paren-mode t)
  (error
   (message "Cannot show parens %s" (cdr err))))
;;;

;;; This moves the mouse pointer out of my way when I type.
;(condition-case err
;    (mouse-avoidance-mode 'banish)
;  (error
;   (message "Cannot use mouse-avoid %s" (cdr err))))
;;;

;;;
(setq auto-save-timeout 1800)
(setq require-final-newline t)
(setq search-highlight t)
(setq compilation-window-height 12)
(setq compilation-ask-about-save nil)
;;;

;;; This makes `apropos' and `super-apropos' do everything that they can.
;;; Makes them run 2 or 3 times slower.  Set this non-nil if you have a
;;; fast machine.
(setq apropos-do-all t)
;;;

;;; CC-Mode stuff, customizes C/C++/ObjC language support.
(setq auto-mode-alist
      (append '(("\\.CC$"  . c++-mode)
                ("\\.cc$" . c++-mode)
                ("\\.cpp$" . c++-mode)
                ("\\.cxx$" . c++-mode)
                ("\\.hpp$" . c++-mode)
                ("\\.hxx$" . c++-mode)
                ("\\.hh$" . c++-mode)
                ("\\.c$"  . c-mode)       ; to edit C code
                ;("\\.h$"  . c++-mode)     ; to edit C++ code
                ("\\.h$"  . c-mode)       ; to edit C code
                ("\\.m$"  . objc-mode)    ; to edit ObjC code
                ("\\.java$" . c++-mode)   ; to edit java code
                ("\\.prolog$" . c++-mode) ; to edit prolog code
                ("\\.pl$" . perl-mode)    ; to edit perl code
                ("\\.make$" . makefile-mode)  ; to edit makefiles
                ("\\makefile$" . makefile-mode) ; to edit makefiles
                ("\\CONFIG$" . makefile-mode) ; to edit "CONFIG" files
                ("cshrc" . shell-script-mode) ; to edit .cshrc related files
                ("emacs" . emacs-lisp-mode) ; to edit emacs related files
                ("\\.csh$" . shell-script-mode) ; to edit c-shell scripts
                ("\\.tar$" . tar-mode)  ; to edit tarfiles
                ) auto-mode-alist))
;;;

;;;
;;; Enable some commands we need.
(put 'narrow-to-region 'disabled nil)
(put 'downcase-region 'disabled nil)
(put 'upcase-region 'disabled nil)
;;;

;;;Setting this to t makes scrolling faster, but may momentarily present
;;;unfontified areas when you scroll into them.
;(setq lazy-lock-defer-driven t)
(setq lazy-lock-defer-on-scrolling t)
;;;

;;;This enables archive browsing and editing.
(setq auto-mode-alist
      (cons '("\\.\\(arc\\|zip\\|lzh\\|zoo\\)\\'" . archive-mode)
            auto-mode-alist))
;;;

;;;This enables automatic resizing of the minibuffer when its contents
;;;won't fit into a single line.
(condition-case err
    (resize-minibuffer-mode 1)
  (error
   (message "Cannot resize minibuffer %s" (cdr err))))
;;;

;;; Ps-Print customization.
(setq ps-header-lines 3)
(setq ps-paper-type 'ps-a4)
;;;

;;;
(setq circulate-include "")
;;; C-x C-e runs the command eval-last-sexp:

;;;
(setq Info-additional-directory-list
      '("c:/djgpp/info" "~/info"))
;;;

;;;
;; (setq blink-matching-paren-on-screen t)
;;;


;;; end - various setups
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; font-lock section

;;; Set up support for lazy-lock
(setq font-lock-support-mode '((t . lazy-lock-mode)))
;;;

;;; Set the maximum buffer size for coloured text to unlimited
(setq-default font-lock-maximum-size nil)
;;;

;;;
;; (setq lazy-lock-defer-driven nil)
(setq lazy-lock-defer-on-scrolling t)

;;;--------------------------------
;;; face font-lock
;;;
;;; [colors] gold,orange,gray,cyan,magenta,brown,pink,light*,dark*
;;;-------------------------------
;;; where FACE should be one of the face symbols, and the subsequent element
;;; items should be the attributes for the corresponding Font Lock mode
;;; faces.  Attributes FOREGROUND and BACKGROUND should be strings (default
;;; if nil), while BOLD-P, ITALIC-P, and UNDERLINE-P should specify the
;;; corresponding face attributes (yes if non-nil).
;;;
;;; (FACE FOREGROUND BACKGROUND BOLD-P ITALIC-P UNDERLINE-P)

(set-face-foreground 'default           "white")
(set-face-background 'default           "black")

(set-foreground-color "slategray")
(set-background-color "black")

(set-face-foreground 'region "wheat")
(set-face-background 'region  "darkslategray")
(set-face-italic-p 'region t)

(set-face-foreground 'highlight  "red")
(set-face-background 'highlight  "Blue4")

(set-face-foreground 'secondary-selection "yellow")
(set-face-background 'secondary-selection "MidnightBlue")

(set-cursor-color "red")

(set-face-foreground 'modeline          "gold")
(set-face-background 'modeline          "black")

(set-face-foreground 'show-paren-match-face "white")
(set-face-background 'show-paren-match-face "blueviolet")
(set-face-bold-p 'show-paren-match-face t)

(set-face-foreground 'show-paren-mismatch-face "black")
(set-face-background 'show-paren-mismatch-face "yellow")
(set-face-italic-p 'show-paren-mismatch-face t)

(setq font-lock-face-attributes
      '(
        (font-lock-comment-face        "SteelBlue" nil nil 1 nil)
        (font-lock-string-face         "cyan" nil nil 1 nil)
        (font-lock-keyword-face        "darkorange"  nil nil nil nil)
        (font-lock-type-face           "yellow" nil nil nil nil)
        (font-lock-function-name-face  "DeepSkyBlue"  nil nil nil 1)
        (font-lock-variable-name-face  "magenta" nil nil nil nil)
        (font-lock-reference-face      "blue" nil nil nil nil)
        (font-lock-builtin-face        "lightblue" nil nil nil nil)
        (font-lock-constant-face       "cornflowerblue" nil nil nil nil)
        (font-lock-warning-face        "red" nil nil nil nil)))
;;;

;;; Turn on font-locking for various modes.
;; (global-font-lock-mode '(shell-script-mode
;;                          fundamental-mode
;;                          text-mode
;;                          makefile-mode))
;;;

;;;
(global-font-lock-mode t)
;;;

;;; Font-locking faces set-up
(setq font-lock-maximum-decoration t)
;; (setq font-lock-maximum-size
;;       (if font-lock-maximum-decoration (* 70 1024) (* 150 1024)))
;;;


;;; end - font-lock section
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; load/require section

;;; Load get-includes.el
(require 'get-includes)
;;;

;;; Load misc.el
(require 'misc)
;;;

;;; Load ctypes
;; (require 'ctypes)
;; (ctypes-auto-parse-mode 1)
;;;

;;; Load fm.el
(require 'fm)
;;;

;;; Draws "boxed comments"
(require 'box-comments)
;;;

;;; Load tar-mode
(autoload 'tar-mode "tar-mode")
;;;

;;; Load cwarnl.el
;; (require 'cwarn)
(add-hook 'c-mode-common-hook
          (function (lambda ()
                      (require 'cwarn)
                      (global-cwarn-mode 1))))
;;;

;;; Load Shell-toggle.el (autoload 'shell-toggle "shell-toggle"
"Toggles between the *shell* buffer and whatever buffer you are
editing."  t) (autoload 'shell-toggle-cd "shell-toggle" "Pops up a
shell-buffer and insert a \"cd <file-dir>\" command." t) ;;;

;;; Load vkill.el
(autoload 'vkill "vkill" nil t)
(autoload 'list-unix-processes "vkill" nil t)
;;;

;;; Load dired-tar.el
(require 'dired-tar)
;;;

;;; Turn on highlighting for Dired (C-x d) and list-buffers (C-b)
(require 'linemenu)

(defadvice list-buffers (after highlight-line activate)
  (save-excursion
    (set-buffer "*Buffer List*")
    (linemenu-initialize)))
(defadvice electric-buffer-menu-looper (after highlight-line activate)
  (linemenu-highlight-current-line))
(add-hook 'dired-after-readin-hook 'linemenu-initialize)
(add-hook 'electric-buffer-menu-hook 'linemenu-initialize)
;;;
   
;;; Load mic-paren.el
(require 'mic-paren)  
(paren-activate)
;;;

;;; Load advice and tabify
;; (require 'advice)
;; (require 'tabify)
;; 
;; (defadvice c-indent-line (after indent-oneline-comment activate compile)
;;   "Indent a oneline comment to the right of code.  When this function
;; is called, point is always at the beginning of the line."
;;   (save-excursion
;;     (let* ((begin (point))
;;            (end (progn (end-of-line) (point))))
;;       (goto-char begin)
;;       (if (re-search-forward "[^ }/]\\([ \t]*\\)/[/*]+.*" end t)
;;           (let ((anchor (match-beginning 1)))
;;             (delete-region anchor (match-end 1))
;;             (let ((len (- comment-column (- anchor begin))))
;;               (goto-char anchor)
;;               (insert-char ?  (- len tab-width 1))
;;               (and personal-use-tabs
;;                    (tabify anchor (point)))))))))
;;;

;;;Load align.el
(autoload 'align "align" nil t)
;;;

;;; Enables completion of recently used words
;;; Meta Return cycles through completions
;; (require 'completion)
;; (initialize-completions)
;;;

;;; Loads scroll-inplace.el
(require 'scroll-in-place)
;;;

;;;This loads a package which enables automatic scrolling when long
;;;lines are truncated.
(condition-case err
    (require 'auto-show)
  (error
   (message "Cannot auto-scroll long lines %s" (cdr err))))
;;;

;;;This installs the `saveplace' package and defines where the places
;;;in visited files are saved between sessions.
(condition-case err
    (require 'saveplace)
  (error
   (message "Cannot save places %s" (cdr err))))
;(setq-default save-place t)    ; place saved in all files
;(setq save-place-file    "~/_places.sav")
;;;

;;;Load comment.el
(autoload 'comment-out-region "comment" nil t)
;;;

;;;
(autoload 'ifdef:ifdef-region "ifdef" "ifdef your code" t)
(autoload 'ifdef:ifndef-region "ifdef" "ifdef your code" t)
(autoload 'ifdef:ifdef-else-region "ifdef" "ifdef your code" t)
(autoload 'ifdef:if-region "ifdef" "ifdef your code" t)
(autoload 'ifdef:if-else-region "ifdef" "ifdef your code" t)
;;;

;;; Load archive mode
(autoload 'archive-mode "arc-mode" "Major mode for editing archives." t)
;;;

;;; This enables automatic saving of the Emacs desktop configuration into
;;; a file called `emacs.desktop'.  Should be as near the .emacs file end
;;; as possible.
(condition-case err
    (progn
      (load "desktop")
      (desktop-load-default)
      (if (file-exists-p (concat "./" desktop-basefilename))
          (desktop-read)))
  (error
   (message "Cannot save desktop %s" (cdr err))))
(setq desktop-missing-file-warning nil)
;;;

;;; end - load/require
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; mode stuff (various different modes)

;;; Set the default start up mode
(text-mode)
;(setq default-major-mode 'text-mode)
;;;

;;; This defines and sets up my customized c-mode indentation.
(defun my-c-stuff ()
  "My own C stuff.  Swaps RET and LFD (^J) and customizes
indentation style and dynamic abbrevs."
  (modify-syntax-entry ?_ "w")

  (c-set-offset 'defun-open 0 nil)
  (c-set-offset 'defun-block-intro 3 nil)  
  (c-set-offset 'statement 0 nil)
  (c-set-offset 'statement-block-intro 3 nil)
  (c-set-offset 'substatement 3 nil)
  (c-set-offset 'statement-case-intro 3 nil)
  (c-set-offset 'statement-case-open 3 nil)
  (c-set-offset 'inclass 5 nil)
  (c-set-offset 'case-label 3 nil)
  (c-set-offset 'inline-open 0 nil)
  (c-set-offset 'access-label -2 nil)
  (c-set-offset 'label 10 nil)

  (set (make-local-variable 'c-indent-level) 0)
  (setq c-electric-pound-behavior '(alignleft))
  (setq c-progress-interval 8);; or else I tend to think it crashed
  (setq c-macro-shrink-window-flag t)
  (setq c-macro-prompt-flag t)
  (setq c-toggle-auto-state t)
  (set (make-local-variable 'dabbrev-case-fold-search) nil)
  (set (make-local-variable 'dabbrev-case-replace) nil)
  (define-key c-mode-map "\C-c %" 'match-paren)
  (define-key c-mode-map "\C-j" 'newline)
  (define-key c-mode-map "\C-m" 'newline-and-indent)
  (setq c-tab-always-indent t)
  (setq c-basic-offset 0))
;;;
(add-hook 'c-mode-common-hook 'my-c-stuff)
(add-hook 'c-mode-common-hook 'turn-on-font-lock)
;;;

;;; Bind RET to newline-and-indent for c and c++
(add-hook 'c-mode-common-hook
          '(lambda ()
             (local-set-key "\C-m" 'newline-and-indent)))
;;;


;;; Change the face color for #define (font-lock-add-keywords 'c-mode
'(("^#[ ]*define[ ]+\\(\\sw+\\)\\s-*$" 1 font-lock-constant-face)))
;;;

;;; define extra C types to font-lock
(setq c-font-lock-extra-types
      (append
       '("CHAR" "BOOL" "BYTE" "SOCKET" "boolean" "UINT" "UINT16"
"UINT32" "ULONG" "FLOAT" "INT" "INT16" "INT32""uint" "ulong" "string"
"BOOLEAN" "\\sw+_T")
       c-font-lock-extra-types))
;;;

;;; define extra C++ types to font-lock
(setq c++-font-lock-extra-types
      (append
       c-font-lock-extra-types
       c++-font-lock-extra-types))
;;;

;;; Hungry state
;; (add-hook 'c-mode-common-hook
;;           (function
;;            (lambda ()
;;              (c-toggle-hungry-state t))))
;;;

;;; THIS add-hook does a carridge return after a brackets and semi-
;;;colons.
;; (add-hook 'c-mode-common-hook
;;           (function
;;            (lambda ()
;;              (c-toggle-auto-state 1))))
;;;

;;; Makefile stuff
(defun my-makefile-stuff ()
  "My own makefile-mode"
  
  (setq makefile-tab-after-target-colon t)
  )
;;;
(add-hook 'makefile-mode-hook 'my-makefile-stuff)
(add-hook 'makefile-mode-hook 'turn-on-font-lock)
;;;

;;; Perl stuff
(defun my-perl-stuff ()
  "My own Perl-mode."
  
  (setq perl-indent-level 3)
  (setq perl-continued-statement-offset 3)
  (setq perl-continued-brace-offset -3)
  (setq perl-brace-offset 0)
  (setq perl-brace-imaginary-offset 0)
  (setq perl-label-offset 0)

  (setq perl-tab-to-comment t)
  (setq perl-tab-always-indent t))
;;;
(add-hook 'perl-mode-hook 'my-perl-stuff)
(add-hook 'perl-mode-hook 'turn-on-font-lock)
;;;

;;;
(defun my-textmode-stuff ()
  "My own Text-Mode routine. Customizes dynamic abbrevs."
  (set (make-local-variable 'dabbrev-case-fold-search) t)
  (set (make-local-variable 'dabbrev-case-replace) t)
  (set (make-local-variable 'dabbrev-upcase-means-case-search) t))
;;;
(add-hook 'text-mode-hook 'turn-on-auto-fill)
(add-hook 'text-mode-hook 'my-textmode-stuff)
;;;

;;;
(defun my-elisp-stuff ()
  "My own Emacs Lisp stuff.  Currently remaps ^J and ^M and customizes
dynamic abbrevs."
  (set (make-local-variable 'dabbrev-case-fold-search) nil)
  (set (make-local-variable 'dabbrev-case-replace) nil)
  (define-key emacs-lisp-mode-map "\C-j" 'newline)
  (define-key emacs-lisp-mode-map "\C-m" 'newline-and-indent))
;;;
(add-hook 'emacs-lisp-mode-hook 'my-elisp-stuff)
(add-hook 'emacs-lisp-mode-hook 'turn-on-font-lock)
;;;

;;;
(defun my-info-stuff ()
  "My own Info stuff.  Currently sets faces and some variables."
  (set-face-foreground 'info-xref     "yellow")
  (set-face-background 'info-xref     "gray")
  (set-face-foreground 'info-node     "red")
  (set-face-background 'info-node     "gray")
  (setq automatic-footnotes "On"))

(add-hook 'Info-mode-hook 'my-info-stuff)
(setq auto-mode-alist
      (append '(("\\.txh$"  . texinfo-mode))
              auto-mode-alist))
;;;

;;;
(defun my-texinfo-stuff ()
  "My own Texinfo mode customizations."
  (set (make-local-variable 'dabbrev-case-fold-search) nil)
  (set (make-local-variable 'dabbrev-case-replace) nil)
  (define-key emacs-lisp-mode-map "\C-j" 'newline)
  (define-key emacs-lisp-mode-map "\C-m" 'newline-and-indent))

(add-hook 'texinfo-mode-hook 'my-texinfo-stuff)
(add-hook 'texinfo-mode-hook 'turn-on-font-lock)
(add-hook 'change-log-mode-hook 'turn-on-font-lock)
;;;

;;; Electric Buffer Menu mode customization.
(defun my-ebuf-stuff ()
  "My own Electric Buffer Menu stuff.  Currently binds some
convenience keys."
  (define-key electric-buffer-menu-mode-map [up] 'previous-line)
  (define-key electric-buffer-menu-mode-map [down] 'next-line)
  (define-key electric-buffer-menu-mode-map [next] 'scroll-up)
  (define-key electric-buffer-menu-mode-map [previous] 'scroll-down)
  (define-key electric-buffer-menu-mode-map [left] 'scroll-right)
  (define-key electric-buffer-menu-mode-map [right] 'scroll-left))

(add-hook 'electric-buffer-menu-mode-hook 'my-ebuf-stuff)
;;;

;;;This fontifies the compilation buffer when compilation exits.
(defun my-compilation-finish-function (buf msg)
  "This is called after the compilation exits.  Currently just
highlights the compilation messages."
  (save-excursion
    (set-buffer buf)
    (font-lock-fontify-buffer)))
;;;

;;;
(defun my-compilation-mode-stuff ()
  "My own compilation set up"
  (setq compilation-finish-function 'my-compilation-finish-function)
  (setq compilation-window-height 20)
  (setq compilation-ask-about-save t)
  (setq compilation-read-command t)
  (setq compile-auto-highlight t)
  (eetq compile-command "make"))
(add-hook 'compilation-mode-hook 'my-compilation-mode-stuff)
(add-hook 'compilation-mode-hook 'fm-start)
;;;

;; DIRED MODE (add-hook 'dired-mode-hook '(lambda() (define-key
;; dired-mode-map [delete] 'dired-flag-file-deletion) (define-key
;; dired-mode-map [return] 'dired-find-file-other-window) (define-key
;; dired-mode-map [C-down-mouse-1]
;; 'dired-mouse-find-file-other-window))) ;;;

;;; This installs `dired-x.el' when `dired' is first invoked, and also
;;; replaces `find-file' with `dired-x-find-file'.
(add-hook 'dired-load-hook
          (function (lambda ()
                      (setq dired-x-hands-off-my-keys nil)
                      (load "dired-x")
                      ;; Set dired-x global variables here.  For example:
                      ;; (setq dired-guess-shell-gnutar "gtar")
                      (setq dired-local-variables-file "_dired")
                      (setq dired-no-confirm
                            '(byte-compile chmod compress copy load move
                                           print shell symlink uncompress)))))
;;;


;;; Dired-mode. If a directory is on the current line, replace the current
;;; dired buffer with one containing the contents of the directory.
;;; Otherwise, invoke `dired-find-file' on the file. 
(defun dired-follow-file ()
  "In dired, visit the file or directory on this line.
If a directory is on the current line, replace the current
dired buffer with one containing the contents of the directory.
Otherwise, invoke `dired-find-file' on the file."
  (interactive)
  (let ((filename (dired-get-filename)))
    ;; if the file is a directory, replace the buffer with the
    ;;  directories contents
    (if (file-directory-p filename)
        (find-alternate-file filename)
      ;; otherwise simply perform a normal `dired-find-file'
      (dired-find-file))))

;;; Define some dired keys
(add-hook 'dired-mode-hook
          (lambda ()
            (local-set-key "\C-m" 'dired-follow-file)
            (local-set-key "e" 'dired-follow-file)
            (local-set-key "f" 'dired-follow-file)
            (define-key dired-mode-map [left] 'dired-up-directory)
            ;;(define-key dired-mode-map [right] 'dired-advertised-find-file)))
            (define-key dired-mode-map [right] 'dired-follow-file)))
;;;

;;; Set dired-x buffer-local variables
(add-hook 'dired-mode-hook
  (function
   (lambda ()
     ;; Set dired-x buffer-local variables here.  For example:
     ;; (setq dired-omit-files-p t)
     (eval-after-load "dired"
       '(progn
          ;; (defun my-dired-move-to-filename (&optional raise-error eol)
          ;;   (or eol (setq eol (progn (end-of-line) (point))))
          ;;   (beginning-of-line)
          ;;   (forward-char 57)
          ;;   (if (and (> (point) eol) raise-error)
          ;;       (error "No file on this line")))
          
          ;; (defun my-dired-move-to-date (&optional raise-error eol)
          ;;   (or eol (setq eol (progn (end-of-line) (point))))
          ;;   (beginning-of-line)
          ;;   (forward-char 44)
          ;;   (if (and (> (point) eol) raise-error)
          ;;       (error "No file on this line")))
          
          (defun my-dired-move-to-date (&optional raise-error eol)
            (dired-move-to-filename raise-error eol)
            (backward-char 13))
          
          ;; (defun my-dired-move-to-size (&optional raise-error eol)
          ;;   (or eol (setq eol (progn (end-of-line) (point))))
          ;;   (beginning-of-line)
          ;;   (forward-char 31)
          ;;   (if (and (> (point) eol) raise-error)
          ;;       (error "No file on this line")))
          
          (defun my-dired-move-to-size (&optional raise-error eol)
            (dired-move-to-filename raise-error eol)
            (backward-char 25))
          
          (setq dired-font-lock-keywords
                (list
                 ;; Size
                 '("^. -"
                   (" *\\([0-9]+\\).*" (my-dired-move-to-size) nil
                    (1 font-lock-string-face)))
                 
                 ;; Date
                 '("^. [-dlcbs]"
                   ("\\(............\\).*" (my-dired-move-to-date) nil
                    (1 font-lock-type-face)))

                 ;; Directories
                 '("^. d"
                   (0 font-lock-warning-face)
                   (" *\\([0-9]+\\).*" (my-dired-move-to-size) nil
                    (1 font-lock-warning-face))
                   (".+" (dired-move-to-filename) nil
                    (0 font-lock-warning-face))
                   ("[#.][^.].*" (dired-move-to-filename) nil
                    (0 font-lock-warning-face))
                   )

                 ;; Omitted files
                 '(" [#.][^.].*$"            ; obtained from dired-omit-files
                   ("[#.][^.].*" (dired-move-to-filename) nil
                    (0 font-lock-builtin-face)))
                 
                 ;; Symbolic links
                 (list dired-re-sym          ; "^. [0-9 \t]*l"
                       '(0 font-lock-constant-face)
                       '(".+" (dired-move-to-filename) nil
                         (0 font-lock-constant-face)))
                 
                 ;; Backup and autosave files
                 '("[~%#]$"
                   (".+" (dired-move-to-filename) nil (0 font-lock-comment-face)))
                 '("\\.\\(bak\\)$"
                   (".+" (dired-move-to-filename) nil
                    (0 font-lock-comment-face)))
                 
                 ;; program files (source codes, any languages)
                 '("\\.\\(el\\|elc\\|p[ml]\\|cgi\\)$"
                   (".+" (dired-move-to-filename) nil
                    (0 font-lock-constant-face)))

                 ;; c/c++ source files
                 '("\\.\\(c\\|cc\\|cpp\\)$"
                   (".+" (dired-move-to-filename) nil
                    (0 font-lock-string-face)))

                 ;; c/c++ header files
                 '("\\.\\(h\\|hh\\|hpp\\)$"
                   (".+" (dired-move-to-filename) nil
                    (0 font-lock-variable-name-face)))
                 
                 ;; Zipped files
                 '("\\.\\(gz\\|tgz\\|bz2\\|Z\\|zip\\)$"
                   (".+" (dired-move-to-filename) nil (0 font-lock-type-face)))
                 
                 ;; World permissions
                 ;;'("^..[d-]......\\(r..\\|.w.\\|..[xs]\\) " 1 font-lock-type-face)
                 
                 ;; Write permissions
                 '("^. [d-]....\\(w\\)" 1 font-lock-type-face)
                 '("^. [d-]......\\(.w.\\)" 1 font-lock-type-face)
                 
                 ;; Directory name
                 (list dired-subdir-regexp   ; "^. \\([^\n]+\\)\\(:\\)[\n]"
                       '(1 font-lock-function-name-face)
                       '(1 'bold append))
                 
                 ;; Marked for deletion
                 '("^D"
                   (0 font-lock-warning-face prepend)
                   (".+" (dired-move-to-filename) nil
                    (0 font-lock-warning-face prepend))
                   (".+" (beginning-of-line) nil (0 'bold prepend)))
                 
                 ;; Marked (not for deletion)
                 '("^[^ D\n]"
                   (0 font-lock-warning-face prepend)
                   (".+" (beginning-of-line) nil (0 'bold prepend))))))))))
;;;


;;; end - mode stuff (various different modes)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; misc lisp functions

;;; Make backup files in ~/.backups/ rather than scattered around all
;;; over the filesystem.
;; (defun make-backup-file-name (file-name)
;;   "Create the non-numeric backup file name for `file-name'."
;;   (require 'dired)
;;   (if (file-exists-p "~/.backups")
;;       (concat (expand-file-name "~/.backups/")
;;               (dired-replace-in-string "/" "|" file-name))
;;    (concat file-name "~")))
;;;

;;; kills all back-up files
(defun kill-all-backup-files ()
  (interactive)
  (let ((bufs (buffer-list)))
    (while bufs
      (let ((file (buffer-file-name (car bufs))))
        (when file
          (let* ((dir (file-name-directory file))
                 (dired-buf (find-file-noselect dir)))
            (set-buffer dired-buf)
            (dired-unmark-all-marks)
            (dired-flag-backup-files)
            (let ((dired-no-confirm t)
                  (dired-deletion-confirmer #'(lambda (&rest args) t)))
              (dired-do-flagged-delete t)))))
      (setq bufs (cdr bufs)))))
(add-hook 'kill-emacs-hook 'kill-all-backup-files)
;;;

;;;
(defun previous-window (arg)
  "Switch to previous window."
  (interactive "p")
  (other-window (- arg)))
;;;

;;;
(defun next-buffer ()
  (interactive)
  (bury-buffer)
  (let* ((buffer-number (length (buffer-list)))
         (index 0)
         (target-buffer (nth index (buffer-list))))
    (while (and (< index (1- buffer-number))
                (not (string-match circulate-include (buffer-name target-buffer)))
                (or (eq (string-to-char (buffer-name target-buffer)) 32)
                    (eq (string-to-char (buffer-name target-buffer)) 42)))
      (setq index (1+ index))
      (setq target-buffer (nth index (buffer-list))))
    (and (= index (1- buffer-number))
         (setq target-buffer (nth index (buffer-list))))
    (switch-to-buffer target-buffer)))
;;;



;;; call the align.el package and align region as one block
(defun my-align-entire (pt mk)
  (interactive "r")
  (align pt mk "entire"))
;;;

;;; after uncommenting a region, also re-align it.
(defun my-uncomment-and-align (pt mk)
  (interactive "r")
  (save-excursion
    (comment-out-region pt mk -2)
    (my-align-entire pt mk)))
;;;

;;; Jump to top of current page
(defun joc-top-of-page ()
  "Brings point to the top of the current window"
  (interactive)
  (move-to-window-line 0))
;;;

;;; Jump to bottom of current page
(defun joc-bottom-of-page ()
  "Brings point to the bottom of the current window"
  (interactive)
  (move-to-window-line -1))
;;;

;;; Force the showing of matching paren
(defun match-paren (arg)
  "Go to the matching parenthesis if on parenthesis otherwise insert %."
  (interactive "p")
  (cond ((looking-at "\\s\(") (forward-list 1) (backward-char 1))
        ((looking-at "\\s\)") (forward-char 1) (backward-list 1))
        (t (self-insert-command (or arg 1)))))
;;;

;;; Make a window split show current buffer and scratch
(defun my-split-different-windows ()
  (interactive)
  (split-window)
  (set-window-buffer (next-window (selected-window))
                     (get-buffer-create "*scratch*"))
  (other-window 1))
;;;

;;; Open my .emacs file
(defun my-load-DOTemacs ()
  (interactive)
  (find-file "~/.emacs"))
;;;

;;; Load dired with the contents of my site-lisp directory
(defun my-site-lisp-load ()
  (interactive)
  (find-file "~/site-lisp/"))
;;;

;;;
(defun my-modification-stamp ()
  (interactive)
  (save-excursion
    (if (eq major-mode 'c-mode)
        (insert
         "/**********************************************************************\n\n"
         "**********************************************************************/\n")
      (insert
       "/*---------------------------------------------------------------------\n\n"
       "---------------------------------------------------------------------*/\n"))
    (backward-sentence 1)
    (forward-line 1)
    (insert "LAST MODIFIED ON:  \nLAST MODIFIED BY:  Javier Oviedo")))
;;;

;;; Insert my standard header in the current buffer
(defun my-c-header (course name proj due pur)
  (interactive "sCourse Name: \nsFile Name: \nsProject #: \nsDue Date: \nsPurpose: ")
  (if (eq major-mode 'c-mode)
      (insert
       "/*********************************************************************/\n"
       "/*********************************************************************/\n"
       "/*******  Javier Oviedo  *********************************************/\n"
       "/*******\n\n"
       "*******/\n"
       "/*********************************************************************/\n"
       "/*********************************************************************/\n")
    
    (insert
     "/*-------------------------------------------------------------------*/\n"
     "/*-------------------------------------------------------------------*/\n"
     "/*------  Javier Oviedo  --------------------------------------------*/\n\n"
     "/*------\n\n"
     "------*/\n"
     "/*-------------------------------------------------------------------*/\n"
     "/*-------------------------------------------------------------------*/\n"
     ))
  (backward-word 1)
  (forward-line 2)
  (insert "\nCourse Name:  " course "\n")
  (insert "Project #:    " proj "\n")
  (insert "File Name:    " name "\n")
  (insert "Due Date:     " due "\n\n")
  (insert "Purpose: \n         " pur)
  
  )
;;;

;;; Kill buffers by mode, using a regexp
(defun kill-buffers-by-mode (regex)
  "Kill buffers whose major mode matches regexp"
  (interactive "sMajor mode regexp: ")
  (let ((buflist (buffer-list))
                buf)
        (while buflist
          (setq buf (car buflist))
          (setq buflist (cdr buflist))
          (set-buffer buf)
          (if (string-match regex mode-name)
                  (kill-buffer buf)))))
;;;

;;; Inserts a comment header for a c function definition.
(defun function-header (pur ret)
  "Inserts a comment header for a function definition."
  (interactive "sPurpose: \nsReturns: ")
  (insert
   "/*********************************************************************/\n"
   "/**\n")
  (insert "Purpose: \n")
  (insert "   " pur "\n\n")
  (insert "Returns: \n")
  (insert "   " ret "\n")
  (insert
   "**/\n"
   "/*********************************************************************/\n"
   "\n"))
;;;

;;; Insert a comment header for a c++ in-class method.
(defun method-header (pur ret)
  (interactive "sPurpose: \nsReturns: ")
  (insert
   "/*********************************************************************/\n"
   "/**\n")
  (insert "Purpose: \n")
  (insert "   " pur "\n\n")
  (insert "Returns: \n")
  (insert "   " ret "\n")
  (insert
   "**/\n"
   "/*********************************************************************/\n"
   "\n"))
;;;

;;; Insert a comment header for a c++ class definition.
(defun class-header (name pur)
  (interactive "sClass Name: \nsPurpose: ")
  (insert
   "/*-------------------------------------------------------------------*/\n"
   "/*-------------------------------------------------------------------*/\n"
   "/*--\n"
   "Class Name:  " name "\n\n"
   "Purpose: \n" "   " pur "\n"
   "--*/\n"
   "/*-------------------------------------------------------------------*/\n"
   "/*-------------------------------------------------------------------*/\n"
   "\n\n"))
;;;

;;; If point is in a class definition, return the name of the class.
;;; Otherwise, return nil.
(defun get-classname ()
  "If the point is in a class definition, get the name of the class.  Return
nil otherwise."
  (save-excursion
    (let ((brace (assoc 'inclass (c-guess-basic-syntax))))
      (if (null brace) '()
        (goto-char (cdr brace))
        (let ((class-open (assoc 'class-open (c-guess-basic-syntax))))
          (if class-open (goto-char (cdr class-open)))
          (if (looking-at "^class[ \t]+\\([A-Za-z_][^ \t:{]*\\)")
              (buffer-substring (match-beginning 1) (match-end 1))
            (error "Error parsing class definition!")))))))
;;;

;;; Insert a c++ class definition.
(defun new-class (name)
  "Insert a c++ class definition."
  (interactive "sClass Name: ")
  (insert "class " name "\n{\n   public:\n\n   private:\n};\n"))
;;;

;;; Insert method prototype in current header file and matching
;;; method body in implementation file. 
(defun new-method (rettype proto)
  "Insert a method into the current header file at point. Insert
implementation, too.  This function expects the implementation file to
be named foo.cpp and in the same directory as the current file, foo.h."
  (interactive "sReturn type: \nsPrototype: ")
  (let ((classname (get-classname))
        (c-tab-always-indent t))
    (if (null classname) (message "Not in class definition!")
      (unless (string-equal rettype "") (setq rettype (concat rettype " ")))
      (insert rettype proto ";\n")
      (c-indent-command)
      (save-window-excursion
        (find-file (concat
                    (file-name-sans-extension (buffer-file-name))
                    ".cpp"))
        (end-of-buffer)
        (insert "\n\n\n")
        ;; (insert rettype classname "::" proto "\n{\n}\n")))))
        ;(method-header)
        (insert rettype classname)
        (c-toggle-hungry-state t)
        (delete-backward-char 1)
        (c-toggle-hungry-state nil)
        (insert "::" proto"\n{\n}\n")))))
;;;

;;; Insert a function prototype in current header file and matching
;;; function body in source file.
(defun new-function (rettype proto name)
  "Insert a function into the current header file at point. Insert
implementation, too.  This function can take the source file to be the .c
version of the header or any other source file that the user wants to
specify. Files must be in the same directory, though."
  (interactive "sReturn type: \nsPrototype: \nsSource file: ")
  (setq c-tab-always-indent t)
  (insert rettype " " proto ";\n")
  (c-indent-command)
  (save-window-excursion
    (if (string-equal name "")
        (find-file (concat
                    (file-name-sans-extension (buffer-file-name)) ".c"))
      (find-file name))
    
    (end-of-buffer)
    (insert "\n\n\n")
    (call-interactively 'function-header)
    (insert rettype " " proto"\n{\n}\n")))
;;;

;;; Duplicate current line and paste it in the next line.
(defun dup-line()
  "Copy the current line and insert copy as the following line."
  (interactive)                            
  (save-excursion
    (forward-line 1)
    (copy-from-above-command)
    (insert "\n")))
;;;

;;; Insert flagged debug statement
(defun insert-debug-clause () (interactive)
  "Insert a fresh debugging printf()-statement and position point in it"
  (c-indent-command)
  (insert-string "/* DEBUG */\n")
  (c-indent-command)
  ;; (insert-string "cdgPrintf(CDG_DEBUG, \"\\n\");\n")
  (insert-string "printf(\"I AM TRYING TO DEBUG!!!\\n\");\n")
  (c-indent-command)
  (insert-string "/* DEBUG */\n")
  (c-indent-command)
  (previous-line 2)
  (forward-word 3)
  (forward-char 3))
;;;

;;; Remove flagged debug statements
(defun delete-printfs () (interactive)
  "eliminate sections bounded by /* DEBUG */ <stuff> /* DEBUG */"
  (save-excursion
    (goto-char (point-min))
    (query-replace-regexp 
     "[\t\n ]*/\\* DEBUG \\*/[^/]+/\\* DEBUG \\*/[\t\n ]*\n" 
     "\n\n")))
;;;

;;; Inserts time stamp modification on files that have been changed
;;; on disk
(defun my-time-stamp ()
  (interactive)
  (save-excursion
    (let ()
      (goto-char (point-min))
      (if (eq major-mode 'emacs-lisp-mode)
          (let ()
            (replace-regexp
             "LAST[^/]+Oviedo\n"
             "LAST MODIFIED ON:  \n;;; LAST MODIFIED BY:  Javier Oviedo\n")
            (let ()
              (backward-word 6)
              (forward-char 5)
              (insert(format-time-string "%a %d-%b-%Y %T"))))
      (let ()
        (replace-regexp
         "LAST[^/]+Oviedo\n"
         "LAST MODIFIED ON:  \nLAST MODIFIED BY:  Javier Oviedo\n")
        (let ()
          (backward-word 5)
          (backward-char 1)
          (insert(format-time-string "%a %d-%b-%Y %T"))))))))
;;;

;;; Change the time stamp of the current buffer if the buffer
;;; was modified.
(defun my-save-buffer ()
  (interactive)
  (if (buffer-modified-p)
      (my-time-stamp))
  (save-buffer))
;;;


;;; end - misc lisp functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; key customizations (GLOBAL)


;;;
(global-set-key "\M-z" (function (lambda () (interactive) (scroll-down 1))))
(global-set-key "\C-z" (function (lambda () (interactive) (scroll-up 1))))
(global-set-key "\C-l" (function (lambda () (interactive) (recenter 5))))
(global-set-key [C-kp-divide]
                (function
                 (lambda () (interactive) 
                   (insert-file "~/.comment1"))))

(global-set-key [C-kp-multiply]
                (function
                 (lambda () (interactive) 
                   (insert-file "~/.comment2"))))
;;;

;;; PC Function Keys
(global-set-key [f1] 'delete-other-windows)
(global-set-key [f2] 'split-window-vertically)
(global-set-key [f3] 'indent-region)
(global-set-key [f4] 'bury-buffer)
(global-set-key [f5] 'align)
(global-set-key [f6] 'shell-toggle)
(global-set-key [f7] 'shell-toggle-cd)
(global-set-key [f8] 'insert-file)
(global-set-key [f9] 'compile)
(global-set-key [f10] 'my-header-file)
(global-set-key [f11] 'my-site-lisp-load)
(global-set-key [f12] 'my-load-DOTemacs)
(global-set-key [home] 'beginning-of-buffer)
(global-set-key [end] 'end-of-buffer)
(global-set-key [C-home] 'joc-top-of-page)
(global-set-key [C-end] 'joc-bottom-of-page)
(global-set-key [M-home] 'previous-window)
(global-set-key [M-end] 'other-window)
(global-set-key [DEL] 'delete-char)
(global-set-key [delete] 'delete-char)
(global-set-key [M-kp-divide] 'dabbrev-expand)
(global-set-key [M-right] 'end-of-line)
(global-set-key [M-left] 'beginning-of-line)
(global-set-key [C-prior] 'enlarge-window)
(global-set-key [C-next] 'shrink-window)
(global-set-key [C-minus] 'undo)
(global-set-key [C-backspace] 'backward-kill-word)
(global-set-key [C-delete] 'kill-word)
(global-set-key [C-SPC] 'set-mark-command)
(global-set-key [C-minus] 'undo)
;;;

;;; Misc keys
;; (global-unset-key "\C-x\C-s")
;; (global-set-key "\C-x\C-s" 'my-save-buffer)
(global-set-key "\C-x\C-s" 'save-buffer)

(global-set-key "\C-cd" 'dup-line)
(global-set-key "\C-ci" 'ifdef:ifdef-region)
(global-set-key "\C-cn" 'ifdef:ifndef-region)
(global-set-key "\C-ce" 'ifdef:ifdef-else-region)
(global-set-key [vertical-scroll-bar down-mouse-1] 'scroll-bar-drag)
(global-set-key "\C-cs" 'my-split-different-windows)
(global-set-key "|" 'match-paren)
(global-set-key "\C-cq" 'my-uncomment-and-align)
(global-set-key "\M-[" 'my-align-entire)
(global-set-key "\C-x\C-a" 'my-align-entire)
(global-set-key "\C-cc" 'comment-out-region)
(global-set-key "\C-x\C-k" 'kill-this-buffer)
(global-set-key "\M-l" 'load-library)
(global-set-key "\C-x\C-l" 'goto-line)
(global-set-key "\C-o" 'other-window)
(global-set-key "\M-sr" 'query-replace-regexp)
(global-set-key "\C-b" 'electric-buffer-list)
(global-set-key "\C-xu" 'undo)
(global-set-key "\C-xg" 'goto-line)
(global-set-key "\C-x\C-b" 'electric-buffer-list)
(global-set-key "\M-i" 'indent-relative)
(global-set-key "\C-xy" 'insert-register)
(global-set-key "\M-*" 'query-replace-regexp)
(global-set-key "\C-xt" 'shell-toggle-cd)
(fset 'kill-whole-line [home ?\C-k ?\C-k])
;;;

;;; Dired-x installation and customization.
;;; This makes `dired-jump' and `dired-jump-other-window' work
;;; even *before* `dired' has been loaded.
(define-key global-map "\C-x\C-j" 'dired-jump)
;;;

;; You can say C-x 4 b if you want to specify the buffer for the other
;; window.

;; (define-key global-map [C-f1] 'ifdef:ifdef-region)
;; (define-key global-map [C-S-f2] 'ifdef:ifndef-region)
;; (define-key global-map [C-S-f3] 'ifdef:ifdef-else-region)
;; (define-key global-map [M-S-f4] 'ifdef:if-region)
;; (define-key global-map [M-S-f5] 'ifdef:if-else-region)
;; (global-set-key "\C-cq" (kbd "C-u C-c c")) ;; uncomment region
;; (global-set-key [C-backspace] 'kill-whole-line)
;; (global-set-key "\C-xt" 'shell-toggle)
;; (global-set-key "\C-i" 'iconify-or-deiconify-frame)
;; (global-set-key [end] 'end-of-line)
;; (global-set-key [home] 'beginning-of-line)
;; (global-set-key [f8] 'next-buffer)
;; (global-set-key [f4] 'kill-buffer)
;; (global-set-key [f2] 'save-buffer)
;; (global-set-key [end] 'end-of-line)
;; (global-set-key [home] 'beginning-of-line)
;; (global-set-key [f1] 'info)

;;; end - key customizations (general)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; NEW STUFF --- Still testing, or not tested yet.

;; There are two useful hings that I would like TAB to do:
;;
;; a) indent according to the current major mode's notion of nesting
;;
;; b) complete a word as under readline.
;;
;; I use the text around point as a cue what it is that I want from the
;; editor. Allowance has to be made for the case that point is at the
;; edge of a buffer.
;; (defun indent-or-expand () 
;;   "Either indent according to mode, or expand the word preceding point."
;;   (interactive)
;;   (if (and
;;        (= ?w (char-syntax (char-before)))
;;        (not (= ?w (char-syntax (char-after)))))
;;       (dabbrev-expand nil)
;;     (indent-according-to-mode)))
